]> de.git.xonotic.org Git - xonotic/darkplaces.git/blobdiff - cl_demo.c
win32: add two includes to make sure the constants for file mode and sharing are...
[xonotic/darkplaces.git] / cl_demo.c
index 08f84745425d26fcecda08795abe157d892298bf..6a0f194c5b88834e522cc9b667a501aa5c7a825d 100644 (file)
--- a/cl_demo.c
+++ b/cl_demo.c
@@ -23,7 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 extern cvar_t cl_capturevideo;
 int old_vsync = 0;
 
-void CL_FinishTimeDemo (void);
+static void CL_FinishTimeDemo (void);
 
 /*
 ==============================================================================
@@ -88,8 +88,9 @@ void CL_StopPlayback (void)
        if (cls.timedemo)
                CL_FinishTimeDemo ();
 
-       if (COM_CheckParm("-demo") || COM_CheckParm("-capturedemo"))
-               Host_Quit_f();
+       if (!cls.demostarting) // only quit if not starting another demo
+               if (COM_CheckParm("-demo") || COM_CheckParm("-capturedemo"))
+                       Host_Quit_f();
 
 }
 
@@ -98,7 +99,7 @@ void CL_StopPlayback (void)
 CL_WriteDemoMessage
 
 Dumps the current net message, prefixed by the length and view angles
-====================
+#====================
 */
 void CL_WriteDemoMessage (sizebuf_t *message)
 {
@@ -138,7 +139,7 @@ void CL_CutDemo (unsigned char **buf, fs_offset_t *filesize)
        // restart the demo recording
        cls.demofile = FS_OpenRealFile(cls.demoname, "wb", false);
        if(!cls.demofile)
-               Host_Error("failed to reopen the demo file");
+               Sys_Error("failed to reopen the demo file");
        FS_Printf(cls.demofile, "%i\n", cls.forcetrack);
 }
 
@@ -238,16 +239,21 @@ void CL_ReadDemoMessage(void)
                }
 
                // get the next message
-               FS_Read(cls.demofile, &net_message.cursize, 4);
-               net_message.cursize = LittleLong(net_message.cursize);
-               if(net_message.cursize & DEMOMSG_CLIENT_TO_SERVER) // This is a client->server message! Ignore for now!
+               FS_Read(cls.demofile, &cl_message.cursize, 4);
+               cl_message.cursize = LittleLong(cl_message.cursize);
+               if(cl_message.cursize & DEMOMSG_CLIENT_TO_SERVER) // This is a client->server message! Ignore for now!
                {
                        // skip over demo packet
-                       FS_Seek(cls.demofile, 12 + (net_message.cursize & (~DEMOMSG_CLIENT_TO_SERVER)), SEEK_CUR);
+                       FS_Seek(cls.demofile, 12 + (cl_message.cursize & (~DEMOMSG_CLIENT_TO_SERVER)), SEEK_CUR);
                        continue;
                }
-               if (net_message.cursize > net_message.maxsize)
-                       Host_Error("Demo message (%i) > net_message.maxsize (%i)", net_message.cursize, net_message.maxsize);
+               if (cl_message.cursize > cl_message.maxsize)
+               {
+                       Con_Printf("Demo message (%i) > cl_message.maxsize (%i)", cl_message.cursize, cl_message.maxsize);
+                       cl_message.cursize = 0;
+                       CL_Disconnect();
+                       return;
+               }
                VectorCopy(cl.mviewangles[0], cl.mviewangles[1]);
                for (i = 0;i < 3;i++)
                {
@@ -255,9 +261,9 @@ void CL_ReadDemoMessage(void)
                        cl.mviewangles[0][i] = LittleFloat(f);
                }
 
-               if (FS_Read(cls.demofile, net_message.data, net_message.cursize) == net_message.cursize)
+               if (FS_Read(cls.demofile, cl_message.data, cl_message.cursize) == cl_message.cursize)
                {
-                       MSG_BeginReading();
+                       MSG_BeginReading(&cl_message);
                        CL_ParseServerMessage();
 
                        if (cls.signon != SIGNONS)
@@ -298,7 +304,7 @@ void CL_Stop_f (void)
        }
 
 // write a disconnect message to the demo file
// LordHavoc: don't replace the net_message when doing this
        buf.data = bufdata;
        buf.maxsize = sizeof(bufdata);
        SZ_Clear(&buf);
@@ -306,10 +312,16 @@ void CL_Stop_f (void)
        CL_WriteDemoMessage(&buf);
 
 // finish up
+       if(cl_autodemo.integer && (cl_autodemo_delete.integer & 1))
+       {
+               FS_RemoveOnClose(cls.demofile);
+               Con_Print("Completed and deleted demo\n");
+       }
+       else
+               Con_Print("Completed demo\n");
        FS_Close (cls.demofile);
        cls.demofile = NULL;
        cls.demorecording = false;
-       Con_Print("Completed demo\n");
 }
 
 /*
@@ -323,6 +335,7 @@ void CL_Record_f (void)
 {
        int c, track;
        char name[MAX_OSPATH];
+       char vabuf[1024];
 
        c = Cmd_Argc();
        if (c != 2 && c != 3 && c != 4)
@@ -361,7 +374,7 @@ void CL_Record_f (void)
 
        // start the map up
        if (c > 2)
-               Cmd_ExecuteString ( va("map %s", Cmd_Argv(2)), src_command);
+               Cmd_ExecuteString ( va(vabuf, sizeof(vabuf), "map %s", Cmd_Argv(2)), src_command, false);
 
        // open the demo file
        Con_Printf("recording to %s.\n", name);
@@ -394,6 +407,7 @@ void CL_PlayDemo_f (void)
        char    name[MAX_QPATH];
        int c;
        qboolean neg = false;
+       qfile_t *f;
 
        if (Cmd_Argc() != 2)
        {
@@ -401,6 +415,19 @@ void CL_PlayDemo_f (void)
                return;
        }
 
+       // open the demo file
+       strlcpy (name, Cmd_Argv(1), sizeof (name));
+       FS_DefaultExtension (name, ".dem", sizeof (name));
+       f = FS_OpenVirtualFile(name, false);
+       if (!f)
+       {
+               Con_Printf("ERROR: couldn't open %s.\n", name);
+               cls.demonum = -1;               // stop demo loop
+               return;
+       }
+
+       cls.demostarting = true;
+
        // disconnect from server
        CL_Disconnect ();
        Host_ShutdownServer ();
@@ -408,19 +435,10 @@ void CL_PlayDemo_f (void)
        // update networking ports (this is mainly just needed at startup)
        NetConn_UpdateSockets();
 
-       // open the demo file
-       strlcpy (name, Cmd_Argv(1), sizeof (name));
-       FS_DefaultExtension (name, ".dem", sizeof (name));
        cls.protocol = PROTOCOL_QUAKE;
 
        Con_Printf("Playing demo %s.\n", name);
-       cls.demofile = FS_OpenVirtualFile(name, false);
-       if (!cls.demofile)
-       {
-               Con_Print("ERROR: couldn't open.\n");
-               cls.demonum = -1;               // stop demo loop
-               return;
-       }
+       cls.demofile = f;
        strlcpy(cls.demoname, name, sizeof(cls.demoname));
 
        cls.demoplayback = true;
@@ -435,6 +453,8 @@ void CL_PlayDemo_f (void)
 
        if (neg)
                cls.forcetrack = -cls.forcetrack;
+
+       cls.demostarting = false;
 }
 
 /*
@@ -443,11 +463,14 @@ CL_FinishTimeDemo
 
 ====================
 */
-void CL_FinishTimeDemo (void)
+static void CL_FinishTimeDemo (void)
 {
        int frames;
+       int i;
        double time, totalfpsavg;
        double fpsmin, fpsavg, fpsmax; // report min/avg/max fps
+       static int benchmark_runs = 0;
+       char vabuf[1024];
 
        cls.timedemo = false;
 
@@ -459,9 +482,25 @@ void CL_FinishTimeDemo (void)
        fpsmax = cls.td_onesecondmaxfps;
        // LordHavoc: timedemo now prints out 7 digits of fraction, and min/avg/max
        Con_Printf("%i frames %5.7f seconds %5.7f fps, one-second fps min/avg/max: %.0f %.0f %.0f (%i seconds)\n", frames, time, totalfpsavg, fpsmin, fpsavg, fpsmax, cls.td_onesecondavgcount);
-       Log_Printf("benchmark.log", "date %s | enginedate %s | demo %s | commandline %s | result %i frames %5.7f seconds %5.7f fps, one-second fps min/avg/max: %.0f %.0f %.0f (%i seconds)\n", Sys_TimeString("%Y-%m-%d %H:%M:%S"), buildstring, cls.demoname, cmdline.string, frames, time, totalfpsavg, fpsmin, fpsavg, fpsmax, cls.td_onesecondavgcount);
+       Log_Printf("benchmark.log", "date %s | enginedate %s | demo %s | commandline %s | run %d | result %i frames %5.7f seconds %5.7f fps, one-second fps min/avg/max: %.0f %.0f %.0f (%i seconds)\n", Sys_TimeString("%Y-%m-%d %H:%M:%S"), buildstring, cls.demoname, cmdline.string, benchmark_runs + 1, frames, time, totalfpsavg, fpsmin, fpsavg, fpsmax, cls.td_onesecondavgcount);
        if (COM_CheckParm("-benchmark"))
-               Host_Quit_f();
+       {
+               ++benchmark_runs;
+               i = COM_CheckParm("-benchmarkruns");
+               if(i && i + 1 < com_argc)
+               {
+                       if(atoi(com_argv[i + 1]) > benchmark_runs)
+                       {
+                               // restart the benchmark
+                               Cbuf_AddText(va(vabuf, sizeof(vabuf), "timedemo %s\n", cls.demoname));
+                               // cannot execute here
+                       }
+                       else
+                               Host_Quit_f();
+               }
+               else
+                       Host_Quit_f();
+       }
 }
 
 /*
@@ -298,7 +304,7 @@ void CL_Stop_f (void)
 
 // write a disconnect message to the demo file
-       // LordHavoc: don't replace the net_message when doing this
+       // LordHavoc: don't replace the cl_message when doing this
        buf.data = bufdata;
        buf.maxsize = sizeof(bufdata);